// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "font.h"
#include "hud_renderer.h"
#include <iostream>

// Font width in pixels
#define FONT_SIZE 1024

// Created by FontMapper v0.01
// TextMapper (c) 2001 Linus Tan

const int _Arial_Black_Widths[] = {
 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36,
 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36, 36,
 16, 16, 24, 32, 32, 48, 43, 13, 19, 19, 27, 32, 16, 16, 16, 13,
 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 16, 16, 32, 32, 32, 29,
 36, 37, 37, 37, 37, 35, 32, 40, 40, 19, 32, 40, 32, 45, 40, 40,
 35, 40, 37, 35, 35, 40, 37, 48, 37, 37, 35, 19, 13, 19, 32, 24,
 16, 32, 32, 32, 32, 32, 19, 32, 32, 16, 16, 32, 16, 48, 32, 32,
 32, 32, 21, 29, 21, 32, 29, 45, 32, 29, 27, 19, 13, 19, 32, 36,
 32, 36, 13, 32, 24, 48, 32, 32, 16, 48, 35, 16, 48, 36, 35, 36,
 36, 13, 13, 24, 24, 24, 24, 48, 16, 46, 29, 16, 48, 36, 27, 37,
 16, 16, 32, 32, 32, 32, 13, 32, 16, 38, 19, 32, 32, 16, 38, 24,
 19, 32, 19, 19, 16, 32, 41, 16, 16, 19, 19, 32, 48, 48, 48, 29,
 37, 37, 37, 37, 37, 37, 48, 37, 35, 35, 35, 35, 19, 19, 19, 19,
 37, 40, 40, 40, 40, 40, 40, 32, 40, 40, 40, 40, 40, 37, 35, 32,
 32, 32, 32, 32, 32, 32, 48, 32, 32, 32, 32, 32, 16, 16, 16, 16,
 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 29, 32, 29,
};

// Extended width info only available with TrueType fonts!

// WidthLeading: Specifies the distance to add to the
// current position before drawing the character glyph.

// WidthGlyph: Specifies the width of the drawn portion of
// the character glyph.

// WidthTrailing: Specifies the current position to provide
// white space to the right of the character glyph.

const int _Arial_Black_WidthLeading[] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 3, 1, 1, 2, 2, 2, 2, 3, 1, 5, 3, 3, 1, 3, 0,
 2, 4, 2, 2, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 2,
 0, 0, 4, 2, 4, 4, 4, 2, 4, 4, 1, 4, 4, 4, 4, 2,
 4, 2, 4, 1, 1, 4, 0, 0, 0, 0, 2, 2, 0, 1, 3, 0,
 0, 2, 3, 2, 2, 2, 0, 2, 3, 3,-3, 3, 3, 3, 3, 2,
 3, 2, 3, 1, 1, 3, 0, 0, 1, 0, 2, 0, 4, 1, 2, 0,
-1, 0, 2, 1, 2, 3, 3, 3,-1, 0, 1, 1, 2, 0, 2, 0,
 0, 2, 2, 2, 2, 4, 0, 0, 0, 0, 1, 1, 2, 0, 2, 0,
 0, 3, 2, 3, 2, 0, 4, 2, 0, 1, 1, 3, 3, 1, 1, 0,
 3, 3, 1, 1, 4, 3, 3, 3, 0, 4, 1, 2, 4, 4, 2, 1,
 0, 0, 0, 0, 0, 0,-2, 2, 4, 4, 4, 4, 2, 4, 1, 2,
 0, 4, 2, 2, 2, 2, 2, 3, 0, 4, 4, 4, 4, 0, 4, 3,
 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 3,-1, 0,
 2, 3, 2, 2, 2, 2, 2, 3, 2, 3, 3, 3, 3, 0, 3, 0,
};


const int _Arial_Black_WidthGlyph[] = {
 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24,
 1, 10, 22, 30, 29, 44, 39, 10, 15, 15, 18, 26, 10, 14, 10, 13,
 28, 20, 28, 28, 31, 28, 28, 28, 28, 28, 10, 10, 26, 26, 26, 26,
 36, 37, 31, 33, 31, 29, 26, 36, 32, 11, 27, 36, 26, 37, 32, 36,
 29, 37, 34, 32, 33, 32, 37, 49, 37, 37, 31, 16, 13, 16, 26, 24,
 12, 29, 27, 28, 27, 28, 21, 27, 26, 10, 16, 29, 10, 42, 26, 28,
 27, 27, 20, 26, 20, 26, 29, 45, 31, 30, 23, 18, 6, 18, 27, 24,
 32, 24, 10, 31, 22, 42, 26, 26, 18, 48, 32, 14, 44, 24, 31, 24,
 24, 10, 10, 22, 22, 16, 24, 48, 16, 44, 26, 14, 44, 24, 23, 37,
 1, 10, 28, 29, 27, 33, 6, 28, 16, 36, 17, 27, 26, 14, 36, 24,
 13, 26, 18, 18, 12, 26, 35, 10, 15, 11, 18, 27, 43, 43, 45, 26,
 37, 37, 37, 37, 37, 37, 48, 33, 29, 29, 29, 29, 13, 14, 18, 16,
 35, 32, 36, 36, 36, 36, 36, 26, 40, 32, 32, 32, 32, 37, 29, 27,
 29, 29, 29, 29, 29, 29, 44, 28, 28, 28, 28, 28, 13, 13, 18, 16,
 28, 26, 28, 28, 28, 28, 28, 26, 28, 26, 26, 26, 26, 30, 27, 30,
};


const int _Arial_Black_WidthTrailing[] = {
 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
 15, 3, 1, 1, 1, 2, 2, 1, 1, 3, 4, 3, 3, 1, 3, 0,
 2, 8, 2, 2, 0, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 1,
 0, 0, 2, 2, 2, 2, 2, 2, 4, 4, 4, 0, 2, 4, 4, 2,
 2, 1,-1, 2, 1, 4, 0,-1, 0, 0, 2, 1, 0, 2, 3, 0,
 4, 1, 2, 2, 3, 2,-2, 3, 3, 3, 3, 0, 3, 3, 3, 2,
 2, 3,-2, 2, 0, 3, 0, 0, 0,-1, 2, 1, 3, 0, 3, 12,
 1, 12, 1, 0, 0, 3, 3, 3,-1, 0, 2, 1, 2, 12, 2, 12,
 12, 1, 1, 0, 0, 4, 0, 0, 0, 2, 2, 1, 2, 12, 2, 0,
 15, 3, 2, 0, 3,-1, 3, 2, 0, 1, 1, 2, 3, 1, 1, 0,
 3, 3, 0, 0, 0, 3, 3, 3, 1, 4, 0, 3, 1, 1, 1, 2,
 0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2, 4, 1, 0, 1,
 2, 4, 2, 2, 2, 2, 2, 3, 0, 4, 4, 4, 4, 0, 2, 2,
 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 3, 0,-1, 0,
 2, 3, 2, 2, 2, 2, 2, 3, 2, 3, 3, 3, 3,-1, 2,-1,
};

void GetCharLocation(int& x, int& y, unsigned char c)
{
    x = c%16;
    y = c/16;
}

void GetCharLocation(float& x1, float& y1, float& x2, float& y2, unsigned char c)
{
    int x, y;

    GetCharLocation(x, y, c);
    
    float pixel_size = 1.0f / float(FONT_SIZE);
    
    x1 = (64 * x) * pixel_size;
    y1 = (64 * y) * pixel_size;
    x2 = (64 * x + 63) * pixel_size;
    y2 = (64 * y + 63) * pixel_size;
}

void DisplayText(float x, float y, float size, std::string text,
    TEXTURE_HANDLE font_texture, IGraphicsSystem *igs, COLOUR c)
{
    igs->SetTexture(0, font_texture);

    float x1, y1, x2, y2;

    for (unsigned int ch = 0; ch < text.size(); ch++)
    {
        GetCharLocation(x1, y1, x2, y2, text[ch]);

        x += float(_Arial_Black_WidthLeading[(unsigned char)text[ch]]) * size;

        HUD_Renderer::Instance().Draw2D(x, y, x + 64 * size, y + 64 * size,
            x1, y1, x2, y2, c);

        x += float(_Arial_Black_WidthGlyph[(unsigned char)text[ch]] +
            _Arial_Black_WidthTrailing[(unsigned char)text[ch]]) * size;
    }
}

float GetTextWidth(float size, std::string text)
{
    float w = 0.0f;

    for (unsigned int ch = 0; ch < text.size(); ch++)
    {
        w += float(_Arial_Black_WidthLeading[(unsigned char)text[ch]] +
                    _Arial_Black_WidthGlyph[(unsigned char)text[ch]] +
                    _Arial_Black_WidthTrailing[(unsigned char)text[ch]]) * size;
    }

    return w;
}

// Returns a string truncated to a certain width
std::string FitString(float width, float size, std::string text)
{
    float w = 0.0f;
    
    std::string return_string;

    for (unsigned int ch = 0; ch < text.size(); ch++)
    {
        float char_width = float(_Arial_Black_WidthLeading[(unsigned char)text[ch]] +
                        _Arial_Black_WidthGlyph[(unsigned char)text[ch]]) * size;

        if (w + char_width < width)
        {
            return_string += text[ch];
        }
        
        w += char_width;
        w += float(_Arial_Black_WidthTrailing[(unsigned char)text[ch]]) * size;
    }

    return return_string;
}








