// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "jpeg_loader.h"
#include <stdio.h>
#include <sstream>

// This extern "C" seems to fix some undefined references.
extern "C"
{
#include <jpeglib.h>
}

void load_jpeg(std::string filename, unsigned char *buffer)
{
    // 1. Allocate and initialize a JPEG decompression object.
    struct jpeg_decompress_struct cinfo;
    struct jpeg_error_mgr jerr;

    cinfo.err = jpeg_std_error(&jerr);
    jpeg_create_decompress(&cinfo);

    // 2. Specify the source of the compressed data.

    FILE * infile;

    if ((infile = fopen(filename.c_str(), "rb")) == NULL)
    {
        std::stringstream error;
        error << "Can't open " << filename;
        throw std::string(error.str());
    }
    jpeg_stdio_src(&cinfo, infile);

    // 3. Call jpeg_read_header() to obtain image info.

    jpeg_read_header(&cinfo, TRUE);

    jpeg_calc_output_dimensions(&cinfo);

    //std::cout << filename << std::endl;
    //std::cout << cinfo.output_width << ' ' << cinfo.output_height <<std::endl;

    // Decompress the image
    JSAMPROW rowptr[1];
    jpeg_start_decompress(&cinfo);
    while ( cinfo.output_scanline < cinfo.output_height )
    {
        rowptr[0] = (JSAMPROW)(unsigned char *)buffer +
            (cinfo.output_scanline * cinfo.output_width * 3);
        jpeg_read_scanlines(&cinfo, rowptr, (JDIMENSION) 1);
    }
    jpeg_finish_decompress(&cinfo);

    jpeg_destroy_decompress(&cinfo);
}

