// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "keyboard.h"

Keyboard::Keyboard()
{
    // Start off as if all keys are up.
    for (int k = 0; k < 256; k++)
    {
        key_state[k] = false;
    }
}

// Returns true if the key corresponding to key is down. Returns false otherwise.
bool Keyboard::KeyPressed(int keycode)
{
    return key_state[keycode];
}

// To be called whenever an SDL_KEYUP or SDL_KEYDOWN event is received by the main loop.
void Keyboard::KeyEventNotify(SDL_KeyboardEvent *keyevent)
{
    int key = 0;

    switch(keyevent->keysym.sym)
    {
        case SDLK_a:
            key = KEY_A;
            break;
        case SDLK_b:
            key = KEY_B;
            break;
        case SDLK_c:
            key = KEY_C;
            break;
        case SDLK_d:
            key = KEY_D;
            break;
        case SDLK_e:
            key = KEY_E;
            break;
        case SDLK_f:
            key = KEY_F;
            break;
        case SDLK_g:
            key = KEY_G;
            break;
        case SDLK_h:
            key = KEY_H;
            break;
        case SDLK_i:
            key = KEY_I;
            break;
        case SDLK_j:
            key = KEY_J;
            break;
        case SDLK_k:
            key = KEY_K;
            break;
        case SDLK_l:
            key = KEY_L;
            break;
        case SDLK_m:
            key = KEY_M;
            break;
        case SDLK_n:
            key = KEY_N;
            break;
        case SDLK_o:
            key = KEY_O;
            break;
        case SDLK_p:
            key = KEY_P;
            break;
        case SDLK_q:
            key = KEY_Q;
            break;
        case SDLK_r:
            key = KEY_R;
            break;
        case SDLK_s:
            key = KEY_S;
            break;
        case SDLK_t:
            key = KEY_T;
            break;
        case SDLK_u:
            key = KEY_U;
            break;
        case SDLK_v:
            key = KEY_V;
            break;
        case SDLK_w:
            key = KEY_W;
            break;
        case SDLK_x:
            key = KEY_X;
            break;
        case SDLK_y:
            key = KEY_Y;
            break;
        case SDLK_z:
            key = KEY_Z;
            break;



        case SDLK_LEFT:
            key = KEY_LEFT;
            break;
        case SDLK_RIGHT:
            key = KEY_RIGHT;
            break;
        case SDLK_UP:
            key = KEY_UP;
            break;
        case SDLK_DOWN:
            key = KEY_DOWN;
            break;
        case SDLK_ESCAPE:
            key = KEY_ESC;
            break;

        case SDLK_RETURN:
            key = KEY_RETURN;
            break;

        case SDLK_PRINT:
            key = KEY_PRINT;
            break;
        case SDLK_SYSREQ:
            key = KEY_SYSREQ;
            break;
        case SDLK_BREAK:
            key = KEY_BREAK;
            break;

        default:
            key = 255;
            break;
    }

    if (keyevent->type == SDL_KEYDOWN)      key_state[key] = true;
    else if (keyevent->type == SDL_KEYUP)   key_state[key] = false;
}
