// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "ode_helper.h"

matrix44 ConvertToMatrix44(const dReal pos[3], const dReal R[12])
{
    matrix44 m;
    m.col[0].x = R[0];
    m.col[0].y = R[4];
    m.col[0].z = R[8];
    m.col[0].w = 0;
    m.col[1].x = R[1];
    m.col[1].y = R[5];
    m.col[1].z = R[9];
    m.col[1].w = 0;
    m.col[2].x = R[2];
    m.col[2].y = R[6];
    m.col[2].z = R[10];
    m.col[2].w = 0;
    m.col[3].x = pos[0];
    m.col[3].y = pos[1];
    m.col[3].z = pos[2];
    m.col[3].w = 1;

    return m;
}

// Given a body and a point in it's local coordinate system (centre of body is origin),
// return that point in the world's coordinate system ( (0,0,0) is origin ).
vector3 ConvertToWorldSpace(const vector3 &local_point, dBodyID body)
{
    dVector3 result;

    dBodyGetRelPointPos(body, local_point.x, local_point.y, local_point.z, result);

    return vector3(result[0], result[1], result[2]);
}
