// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef ODE_THREAD_H
#define ODE_THREAD_H

#include <ode/ode.h>



#define PHYSICS_IN_SEPARATE_THREAD



//! Class that deals with the thread for the physics code.
class OdeThread
{
public:
    //! Returns a reference to the one and only instance of this class.
    static OdeThread& Instance()
    {
        static OdeThread Obj;
        return Obj;
    }

    ~OdeThread();

    // Starts the thread
    void start_thread();

    // Stops the thread
    void stop_thread();

    // Returns the ODE world object
    dWorldID getWorld();
    // Returns the ODE space object
    dSpaceID getSpace();

    // Call this before calling some ODE functions.
    static void ode_mutex_lock();

    // Call this after calling some ODE functions.
    static void ode_mutex_unlock();

    // Updates the physics timestep(s).
    void Update();

private:

    // Constructor is private in this singleton
    OdeThread();
     
    // ODE specific stuff
    dJointGroupID contactgroup;
    dWorldID world;
    dSpaceID space;

    // This is set to true to tell the thread to stop.
    bool thread_exit;

    // Collision callback function
    friend void nearCallback(OdeThread* ot, dGeomID o1, dGeomID o2);

    // Thread function
    friend int ode_thread_function(void*);
};

#endif // ODE_THREAD_H

