// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef VAPOUR_TRAIL_H
#define VAPOUR_TRAIL_H

#include "particle_system.h"

//! VapourTrail inherits from base class particle system
class VapourTrail : public ParticleSystem
{
public:
    // Creates a vapour trail. smoke0 is the texture to use for the vapour trail. 
    // initial_position is the point at which the trail started usually the rocket laucher's 
    // location at that time.
    // velocity is the speed of the smoke source in unit's per second.
    VapourTrail(TEXTURE_HANDLE smoke0, vector3 initial_position, vector3 velocity);

    // For best results this shouldn't be destructed until after the smoke has faded completely.
    ~VapourTrail();

    // Call this while the rocket is still emitting smoke.
    // When the rocket has collided with something keep calling Render (until the smoke clears) 
    // but don't call this anymore.
    void Update();

    // Call this to render the particles.
    void Render();

private:

    // This is set to the time at which the vapour trail started.
    long start_time;

    // Time at which the vapour trail was previously updated.
    long last_updated;

    // Initital position and velocity. velocity is always constant.
    vector3 pos, v;

    // Texture used by smoke particles
    TEXTURE_HANDLE smoke;
};





#endif // VAPOUR_TRAIL_H
